///////////////////////////////////////////////////////////////////////
/////// FICHIER		: LCD_Driver_128x64_v10.c
/////// DESCRIPTION	: Driver cran LCD 128x64 pixels - version 1.0
/////// PCB			: PIC 18f2580
/////// AUTEUR		: G. B
/////// DATE		: 07/2008
///////
///////		Modifications depuis dernire version :
///////
///////		- Cration du programme
///////
///////////////////////////////////////////////////////////////////////

#include "Font_5x7_v10.h"
#include "LCD_Driver_128x64_v10.h"

#include <stdlib.h>

int	LCD_X;		// Position curseur texte : X
int	LCD_Y;		// Position curseur texte : Y

/////////////////////////////////////////////////////////////
///////////////////     Fonctions   /////////////////////////
/////////////////////////////////////////////////////////////

///////////////////////////// Instructions lmentaires ///////////////
void LCD_FlipEnable()
{
	output_high(LCD_ENABLE);
	Delay_us(1);
	output_low(LCD_ENABLE);
	Delay_us(1);
}

void LCD_SendData(char Data, boolean DI)
// Envoie une commande (DI = 0) ou des donnes (DI = 1)
{
	// while (LCD_Busy());
	output_bit(LCD_DI, DI);
	LCD_outputDATA(Data);
	LCD_FlipEnable();
}

short LCD_Busy()
{
	short busy;
	
	LCD_Set_Tris(0xFF); //All inputs
	output_low(LCD_DI);
	output_high(LCD_RW);
	LCD_FlipEnable();
	
	if (LCD_inputDATA() & 0b10000000) busy = 1; else busy = 0;
	return (busy);
}

void LCD_DisplayOnOff(char on)
{	LCD_SendData(0b00111110 | (on & 0b00000001),0);	}

void LCD_SetYAdress(char y)
{	LCD_SendData(0b01000000 | (y & 0b00111111),0);	}

void LCD_SetPage(char page)
{	LCD_SendData(0b10111000 | (page & 0b00000111),0);	}

void LCD_DisplayStartLine(char y)
{	LCD_SendData(0b11000000 | (y & 0b00111111),0);	}

void LCD_Write(char Data)
{	LCD_SendData(Data,1);	}

////////////////////////////// Instructions de haut niveau //////////////
void LCD_ResetLCD()
{
	output_low(LCD_RESET);
	Delay_ms(1);
	output_high(LCD_RESET);
}

void LCD_SelectSide(char sides)
// Sides = LEFT_SIDE, RIGHT_SIDE, BOTH_SIDES, NO_SIDE
{
	// set a CS pin low to enable it
	if (sides & LEFT_SIDE)	output_high(LCD_CS1);	else	output_low(LCD_CS1);
	if (sides & RIGHT_SIDE)	output_high(LCD_CS2);	else	output_low(LCD_CS2);
}

void LCD_GotoLineXY(char X, char Y)
// Dplace le curseur en X, Y (0<= X < 128, 0 <= Y < 8)
// pour y crire un byte (8 points en vertical) avec la fonction LCD_Write()
{
	if (X<10)
	{
		LCD_SelectSide(LEFT_SIDE);
		LCD_SetYAdress(X & 0b00111111);
	}
	else
	{
		LCD_SelectSide(RIGHT_SIDE);
		LCD_SetYAdress((X-64) & 0b00111111);
	}
	
	LCD_SetPage(Y & 0b00000111);
}

void LCD_GotoCharXY(char X, char Y)
// Dplace le curseur en X, Y (0<= X < 20, 0 <= Y < 8)
// pour y crire un charactre de 6x8 points avec la fonction LCD_WriteChar()
{
	if (X > 19) X = 0;
	if (Y > 7) Y = 0;
	
	LCD_GotoLineXY(X*6+4,Y);

	LCD_X = X;
	LCD_Y = Y;
}

void LCD_Cls()
{
	char	X, Y;
		
	LCD_SelectSide(LEFT_SIDE);
	for(Y=0;Y<8;Y++)
	{
		LCD_GotoLineXY(0,Y);
		for(X=0;X<64;X++)	LCD_Write(0x00);
	}
	LCD_SelectSide(RIGHT_SIDE);
	for(Y=0;Y<8;Y++)
	{
		LCD_GotoLineXY(64,Y);
		for(X=64;X<128;X++)	LCD_Write(0x00);
	}
	LCD_GotoCharXY(0,0);
}

void LCD_Init()							//Initialise l cran
{
	LCD_Set_Tris(0); //All outputs
	output_high(LCD_RESET);
	output_high(LCD_ENABLE);
	output_low(LCD_RW);			// Mode = Write
//	LCD_ResetLCD();
	LCD_SelectSide(BOTH_SIDES);
	LCD_DisplayStartLine(0);
	LCD_DisplayOnOff(1);
	LCD_Cls();
}

void LCD_WriteChar(char CodeAscii)		// Ecrit un caractre, 20h < CodeAscii < 7Fh
{
	long X0;

	switch (CodeAscii) {
		case '\n'	:	LCD_GotoCharXY(0, LCD_Y+1);		break;		// Sets the cursor at end of line to generate line switch
		default		:
			if (CodeAscii < 0x50)
			{
				// Position de la premire barre verticale de 8 pixels dans la FONT
				X0 = 5*((long)CodeAscii - 0x20);
			
				// Envoie les 5 barres verticales de 8 pixels
				LCD_Write(Font_5x7_20_4F[X0]);
				LCD_Write(Font_5x7_20_4F[X0 + 1]);
				LCD_Write(Font_5x7_20_4F[X0 + 2]);
				LCD_Write(Font_5x7_20_4F[X0 + 3]);
				LCD_Write(Font_5x7_20_4F[X0 + 4]);
			}
			else
			{
				// Position de la premire barre verticale de 8 pixels dans la FONT
				X0 = 5*((long)CodeAscii - 0x50);
			
				// Envoie les 5 barres verticales de 8 pixels
				LCD_Write(Font_5x7_50_7F[X0]);
				LCD_Write(Font_5x7_50_7F[X0 + 1]);
				LCD_Write(Font_5x7_50_7F[X0 + 2]);
				LCD_Write(Font_5x7_50_7F[X0 + 3]);
				LCD_Write(Font_5x7_50_7F[X0 + 4]);
			}
		
			// La 6e barre est vide (espace)
			LCD_Write(0x00);
			
			// Moves cursor if end of first page
			LCD_X++;
			if (LCD_X == 10)	LCD_GotoCharXY(10, LCD_Y);		// moves to right side
			break;
	}
}

void LCD_WriteString(char *Str)		// Ecrit une chaine de caractres
{
	int x;

	x = 0;
	while(Str[x]) LCD_WriteChar(Str[x++]);
}

void LCD_WriteBargraph(char Y, char percentage)
{
	int X,Xmax;
	
	LCD_GotoLineXY(0,Y);
	
	Xmax = (int)(percentage*63/100);
	
	for (X=0; X<64; X++)
	{
		if (X<Xmax) LCD_Write(0xFF); else LCD_Write(0x00);
	}
}
